// Testing USB joystick function using Arduino Leonardo
// using MPU6050 as analog x, y and z axis input
// Code written by Pin Chung  
// Sep 22, 2020

#include <Joystick.h>
#include <MPU6050_tockn.h>
#include <Wire.h>

MPU6050 mpu6050(Wire);
Joystick_ Joystick;


#define ResetPin 13
#define Joystick1Pin 9

int lastButtonState=0;
int currentButtonState;

float zeroX=0;
float zeroY=0;
float zeroZ=0;

//upper and lower limit on the angle of the MPU6050 board range.
int upperLimit = 45;
int lowerLimit = -45;

void setup() {
  pinMode(Joystick1Pin, INPUT_PULLUP);
  pinMode(ResetPin, INPUT_PULLUP);
  
  Serial.begin(9600);
  Joystick.begin();

  Wire.begin();
  mpu6050.begin();
  mpu6050.calcGyroOffsets(false); //This is is not a necessary call. works fine without it.

}

void ResetMPU6050(float x, float y, float z){
  //Center the Joystick at current position
  zeroX=x;
  zeroY=y;
  zeroZ=z;
}

void loop() 
{
  mpu6050.update();
   float x=mpu6050.getAngleX();
   float y=mpu6050.getAngleY();
   float z=mpu6050.getAngleZ();
   
  if (!digitalRead(ResetPin)) {
    ResetMPU6050(x,y,z);
    }
    
   currentButtonState=!digitalRead(Joystick1Pin); 
   if (currentButtonState != lastButtonState){
      Joystick.setButton(0, currentButtonState); 
      // 0 is joystick button #1 in Windows 10
      lastButtonState=currentButtonState;  
   }
  
  x-=zeroX;
  y-=zeroY;
  z-=zeroZ;
  
   if (x>upperLimit) x=upperLimit;
   else if (x<lowerLimit) x=lowerLimit;
   if (y>upperLimit) y=upperLimit;
   else if (y<lowerLimit) y=lowerLimit;
   if (z>upperLimit) z=upperLimit;
   else if (z<lowerLimit) z=lowerLimit;


 // Map the angles of x, y and z to joystick position 0 to 1024
 // to reverse the axis, switch the last 2 numbers in the map() function
   Joystick.setXAxis(map(x, lowerLimit,upperLimit, 0,1024));
   Joystick.setYAxis(map(y, lowerLimit,upperLimit, 1024, 0));
   Joystick.setZAxis(map(z, lowerLimit,upperLimit, 1024,0));

}
